package io.github.wulkanowy.ui.modules.more

import android.os.Bundle
import android.text.InputFilter
import android.text.InputType
import android.text.Spanned
import android.view.View
import android.widget.EditText
import androidx.appcompat.app.AlertDialog
import androidx.recyclerview.widget.LinearLayoutManager
import com.google.android.material.dialog.MaterialAlertDialogBuilder
import dagger.hilt.android.AndroidEntryPoint
import io.github.wulkanowy.R
import io.github.wulkanowy.data.repositories.PreferencesRepository
import io.github.wulkanowy.databinding.FragmentMoreBinding
import io.github.wulkanowy.ui.base.BaseFragment
import io.github.wulkanowy.ui.modules.Destination
import io.github.wulkanowy.ui.modules.dashboard.DashboardItem
import io.github.wulkanowy.ui.modules.main.MainActivity
import io.github.wulkanowy.ui.modules.main.MainView
import io.github.wulkanowy.ui.modules.message.MessageFragment
import timber.log.Timber
import javax.inject.Inject

private class AttendancePercentageFilter : InputFilter {
    override fun filter(
        source: CharSequence?,
        start: Int,
        end: Int,
        dest: Spanned?,
        dstart: Int,
        dend: Int
    ): CharSequence? {
        val input = dest.toString() + source.toString()
        val floatRepresentation = input.toFloatOrNull()

        if (floatRepresentation != null && floatRepresentation in 0.0..100.0) return null

        return ""
    }
}

@AndroidEntryPoint
class MoreFragment : BaseFragment<FragmentMoreBinding>(R.layout.fragment_more), MoreView,
    MainView.TitledView, MainView.MainChildView {

    @Inject
    lateinit var presenter: MorePresenter

    @Inject
    lateinit var moreAdapter: MoreAdapter

    @Inject
    lateinit var preferencesRepository: PreferencesRepository

    companion object {
        fun newInstance() = MoreFragment()
    }

    override val titleStringId: Int
        get() = R.string.more_title

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        binding = FragmentMoreBinding.bind(view)
        presenter.onAttachView(this)
    }

    override fun initView() {
        moreAdapter.onClickListener = presenter::onItemSelected
        moreAdapter.onLongClickListener = presenter::onItemHold

        with(binding.moreRecycler) {
            layoutManager = LinearLayoutManager(context)
            adapter = moreAdapter
        }
    }

    override fun onFragmentReselected() {
        if (::presenter.isInitialized) presenter.onViewReselected()
    }

    override fun onFragmentChanged() {
        (parentFragmentManager.fragments.find { it is MessageFragment } as MessageFragment?)
            ?.onFragmentChanged()
    }

    override fun updateData(data: List<MoreItem>) {
        with(moreAdapter) {
            items = data
            notifyDataSetChanged()
        }
    }

    override fun popView(depth: Int) {
        (activity as? MainActivity)?.popView(depth)
    }

    override fun openView(destination: Destination) {
        (activity as? MainActivity)?.pushView(destination.destinationFragment)
    }

    override fun onDestroyView() {
        presenter.onDetachView()
        super.onDestroyView()
    }

    private fun showGradeDialog() {
        val hiddenGrades = preferencesRepository.hiddenGrades

        val grades = requireContext().resources.getStringArray(R.array.hidden_settings_bad_grades)
        val selectedItemsState = grades.map { grade -> hiddenGrades.any { it == grade } }

        MaterialAlertDialogBuilder(requireContext())
            .setTitle(R.string.pref_hidden_settings_bad_grades_title)
            .setMultiChoiceItems(grades, selectedItemsState.toBooleanArray()) { _, _, _ -> }
            .setPositiveButton(android.R.string.ok) { dialog, _ ->
                val selectedState = (dialog as AlertDialog).listView.checkedItemPositions
                val selectedValues = grades.filterIndexed { index, _ -> selectedState[index] }

                Timber.i("Selected hidden grades: $selectedValues")
                presenter.onHiddenGradesSelected(selectedValues)
            }
            .show()
    }

    override fun showHiddenSettings(data: List<DashboardItem.HiddenSettingTile>) {
        val entries = requireContext().resources.getStringArray(R.array.hidden_settings_entries)
        val values = requireContext().resources.getStringArray(R.array.hidden_settings_values)
        val selectedItemsState = values.map { value -> data.any { it.name == value } }

        val attendancePercentage = preferencesRepository.attendancePercentage

        val input = EditText(requireContext()).apply {
            setPadding(40, 20, 40, 20)
            hint = requireContext().getString(R.string.pref_hidden_settings_hint)
            filters = arrayOf(AttendancePercentageFilter())
            inputType = InputType.TYPE_CLASS_NUMBER or InputType.TYPE_NUMBER_FLAG_DECIMAL
        }

        // enable only if attendance modifier is enabled
        val attendanceModifierEnabled = data.any { it == DashboardItem.HiddenSettingTile.ATTENDANCE }
        input.isEnabled = attendanceModifierEnabled

        if (attendancePercentage != null) input.setText(attendancePercentage.toString())

        MaterialAlertDialogBuilder(requireContext())
            .setTitle(R.string.pref_hidden_settings_title)
            .setMultiChoiceItems(entries, selectedItemsState.toBooleanArray()) { dialog, index, _ ->
                // if attendance modifier is enabled, enable text input
                if (index == values.indexOf(DashboardItem.HiddenSettingTile.ATTENDANCE.name))
                    input.isEnabled = (dialog as AlertDialog).listView.checkedItemPositions[index]
            }
            .setView(input)
            .setPositiveButton(android.R.string.ok) { dialog, _ ->
                val selectedState = (dialog as AlertDialog).listView.checkedItemPositions
                val selectedValues = values.filterIndexed { index, _ -> selectedState[index] }
                val inputAttendancePercentage = selectedValues
                    .find { it == DashboardItem.HiddenSettingTile.ATTENDANCE.name }
                    ?.let { input.text.toString().toFloatOrNull() }

                Timber.i("Selected hidden settings: $selectedValues")
                presenter.onHiddenSettingsSelected(selectedValues, inputAttendancePercentage)
            }
            .setNegativeButton(android.R.string.cancel) { _, _ -> }
            .setNeutralButton(R.string.pref_hidden_settings_bad_grades_title) { _, _ -> showGradeDialog() }
            .show()
    }

    override fun restartApp() {
        startActivity(MainActivity.getStartIntent(requireContext()))
        requireActivity().finishAffinity()
    }
}
